#! /usr/bin/env perl
#
# script to run udpmon as client end and log the output
# to a file of format UDP-yyyy-mm.txt in specified directory
#
# r.tasker 27-July-2001
# Richard Hughes-Jones 3 Oct 2001
#
# HISTORY
#
$version = "1.3.8, Jan 31th, 2006";
# Benjamin VIAL <benjamin.vial@urec.cnrs.fr>
# - parameters configuration using a centrally-managed configuration downloaded
#   by glite-netmon-config-update
#
$version = "1.3.3, May 31th, 2005";
# Benjamin VIAL <benjamin.vial@urec.cnrs.fr>
# - only measurements compliant with the specified metric "schema" are sent and produced into R-GMA
#


use POSIX;

use Fcntl;
use Socket;
require "ctime.pl";

$| = 1;

my $GLITE_LOCATION=$ENV{GLITE_LOCATION};
my $GLITE_LOCATION_LOG=$ENV{GLITE_LOCATION_LOG};

#################### Customize me! ##############################

# directory where the data is stored
$udp_data_dir = "$GLITE_LOCATION/var/glite-udpmon";

# full path of the directory with the udp client program
$udp_dir = "$GLITE_LOCATION/bin/";

# client program
$udp_client = "udpmon_bw_mon";

# location of glite-netmon-rgma's communication FIFO
$rgma_fifo = "$GLITE_LOCATION/var/glite-netmon2rgma/glite-netmon2rgma-fifo";

# Unique identifying name for the tool
$tool_name = "udpmon";

# Log file
$logfile = "$GLITE_LOCATION_LOG/e2emonit-udpmon.log";

$localhost = "127.0.0.1";

###################################################################

# Organise a file handle for the output
($min, $hour, $day, $month, $year) = (localtime()) [1,2,3,4,5];
$month++;
$year += 1900;
local($udp_data_fn) = sprintf "$udp_data_dir/udp-%4d-%2.2d\.txt", $year, $month;

# try to open the file assuming it exists (this will be true most of the time, so we try it first)
if (!sysopen UDPDATA, $udp_data_fn, O_RDWR | O_EXCL | O_APPEND) {
    #
    # file doesn't exist, so try creating it
    if (!(sysopen(UDPDATA, $udp_data_fn, (O_RDWR | O_EXCL | O_CREAT)))) {
	#
	# file appeared sometime between the first and second sysopen calls,
	# try opening it again
	#
	if (!sysopen UDPDATA, $udp_data_fn, O_RDWR | O_EXCL | O_APPEND){
	    #
	    # error!!
	    #
	    print STDERR "Can't open $udp_data_fn: $!\n";
	    exit;
	}
    }
}

###################################################################
# Scanning for argument / Get configfile(.txt)

my $self = $0;
$self =~  s!^.*/!!;
if (@ARGV != 1)
{
    die "Message at: " . `date` . "...Usage:  perl $self configfile(.txt)\n ";
}

open (CONFIG_LIST, "<$ARGV[0]") or die "Message at: " . `date` . "...Can't open $ARGV[0]: $!\n";
@config_list_lines=<CONFIG_LIST>;
close(CONFIG_LIST);

###################################################################
# Configure parameters from e2emonit_configuration.txt

while ($_=shift(@config_list_lines)) {

        next if(/^\s$/);
        if(/^List/){
                $list_cursor = 1;
                next;
        }
        if (/^\#/){
                $list_cursor = 0;
                next;
        }
        if (/^UDPmon/){
                $tool_cursor = 1;
                next;
        }
        chop();
        # Get monitoring list
        unless($list_cursor != 1){

                        push(@list,$_);

        }
        # Get parameters
        unless($tool_cursor != 1){

                # Check parameters format, get value
                if($_ =~ /^[\-]\D{1}\d*$/){

                        $list_parameter="$list_parameter $_";
                }
                last if(/^End/);
        }

}

# Initialise RGMA
$host_name=`/bin/hostname -f`;
chomp($host_name);
&initialise_rgma(scalar(@list),$host_name);
    
foreach $dest_name (@list) {

    # Get IP adress
    ($true_name, $aliases, $addrtype, $addrlength, @addrs) = gethostbyname($dest_name);

    if (!defined $true_name && !defined $dest_ip) {
      die " Message at: " . `date` . "...Can't resolve $dest_name to IP address\n";
    }
    elsif (defined $true_name) {
      ($a, $b, $c, $d) = unpack('C4', $addrs[0]);
      $dest_ip = "$a.$b.$c.$d";
      #print "Destination IP: " . $dest_ip . "\n";
    }

    # define client udpmon command
    $udp_cmd =  $udp_dir.$udp_client." -d \$dest_ip $list_parameter -q";
    $_ = $udp_cmd;
    s/"/\\"/g;
    $eval_udp_cmd = eval qq/"$_"/;
    
    $pid = open(UDP, "-|");
    if ($pid == 0) {
	
	# sort out stderr and exec the command
	open(STDERR, ">&STDOUT")  || die "Message at: " . `date` . "...couldn't redirect stderr:$!\n";
	exec $eval_udp_cmd;
    }
    while (<UDP>) {
	
	# collect the output
	$output .= $_;

	# check for and log any error reported by udpmon
	$time=&time2timestamp($time);

	if ($output =~ /Error/) {
	    
	    if (! open(OUTPUT,">>$logfile") ) {
		
                die "Message at: " . `date` . "...Can't open output file $logfile\n";
            }

	    print OUTPUT "----------------------------------------------\n";
            print OUTPUT "Message at: " . `date`;
            print OUTPUT $output;
	    print STDERR "$output";
	    close UDP;
	    close UDPDATA;
            #exit;   	    
	}
    }
    close UDP;
    print UDPDATA "$output\n";
    
    #Send to glite-netmon-rgma
    if (!($output=~/Error/)) {
	my $pkt_loss;
	$output=~s/^\s*(.*?)\s*$/$1/;  # remove leading and trailing white space;
	my @row = split(/\s*;\s*/,$output);
	if ($#row > 26) { # V3.2 adds %loss as field 15
	    $pkt_loss = splice(@row,16,1);
	}
	my @indices = (0,1,3,5,6,10,13,18,19,21);
	my ($ctime, $src_name, $dest_name, $udp_size, $num_sent, $wait_time,
	    $num_recvd, $user_rate_recvd, $wire_rate_recvd, $jitter_mean)
	    = (@row)[(@indices)];
	if (!defined $pkt_loss) {
	    $pkt_loss = (100.0*(1-$num_recvd/$num_sent));
	}
	&produce_it($ctime, $src_name, $dest_name, $udp_size, $num_sent, $wait_time, $num_recvd, $user_rate_recvd, $wire_rate_recvd, $jitter_mean, $pkt_loss);
    }
    
    $output = "";
}   # end loop over udp cmds

close(UDPDATA);


# this subrouting initialises glite-netmon-rgma for us
sub initialise_rgma {
    my ($count,$host_name) = @_;

    # Don't try to write if it is not a fifo
    return if ! -p $rgma_fifo;

    chomp($count);
    chomp($host_name);


    # Give the server 30 seconds to accept our data, otherwise give up
    # the die will make the *eval* die, but not the current thread
    $SIG{ALRM} = sub{die "timeout"};
	
    eval {
	alarm(30);

	open(SERVER,"> $rgma_fifo") || die "Message at: " . `date` . "...left eval, fifo not openend";
	print SERVER ("addmetric $tool_name udpthroughput $count NMIdSource=$host_name tool=$tool_name\n");
	print SERVER ("addmetric $tool_name onewayipdv $count NMIdSource=$host_name tool=$tool_name\n");
	print SERVER ("addmetric $tool_name udppacketloss $count NMIdSource=$host_name tool=$tool_name\n");
	close(SERVER);
	
	alarm(0);
    };

    $SIG{ALRM} = 'DEFAULT';
}

sub time2timestamp {
    my ($time)=@_;

    my ($seconds, $minutes, $hours, $day_of_month, $month, $year, $wday, $yday, $isdst) = gmtime($time);

    return sprintf("%04d%02d%02d%02d%02d%02dZ",$year+1900,$month+1,$day_of_month,$hours,$minutes,$seconds);
}

# this subroutine produces information into RGMA via glite-netmon-rgma
sub produce_it {
    my ($ctime, $src_ip, $dest_ip, $udp_size, $num_sent, $wait_time, $num_recvd, $user_rate_recvd, $wire_rate_recvd, $jitter_mean, $pkt_loss) = @_;

    # Don't try to write if it is not a fifo
    return if ! -p $rgma_fifo;

    #See if we can get real hostnames
    # my $src_name = gethostbyaddr(pack("C4",split(/\./,$src_ip)),AF_INET);
    my $src_name = gethostbyaddr(inet_aton($src_ip),AF_INET);
    # my $dest_name = gethostbyaddr(pack("C4",split(/\./,$dest_ip)),AF_INET);
    my $dest_name = gethostbyaddr(inet_aton($dest_ip),AF_INET);

    if ($src_name && $dest_name) {

	# Give the server 30 seconds to accept our data, otherwise give up
	# the die will make the *eval* die, but not the current thread
	$SIG{ALRM} = sub{die "timeout"};
	
	eval {
	    alarm(30);

	    open(SERVER,"> $rgma_fifo") || die "Message at: " . `date` . "...left eval, fifo not openend";
	    $ctime = &time2timestamp($ctime);
	    print SERVER ("measurement $tool_name udpthroughput $src_name $dest_name udpmon $udp_size $num_sent $wait_time $ctime $user_rate_recvd $wire_rate_recvd\n");
	    print SERVER ("measurement $tool_name udppacketloss $src_name $dest_name udpmon $udp_size $num_sent $wait_time $ctime $pkt_loss\n");
	    print SERVER ("measurement $tool_name onewayipdv $src_name $dest_name udpmon $udp_size $num_sent $wait_time $ctime $jitter_mean\n");
	    close SERVER;
	    
	    alarm(0);
	};
	
	$SIG{ALRM} = 'DEFAULT';
    }
}
